<?php

namespace Concrete\Package\JlElephpantInfo;

use Concrete\Core\Page\Page;
use Concrete\Core\Page\Single as SinglePage;
use Concrete\Core\Package\Package;

defined('C5_EXECUTE') or die('Access Denied.');

/*
 * John Liddiard (aka JohntheFish) www.c5magic.co.uk
 * This software is licensed under the terms described in the concrete5.org marketplace.
 * Please find the add-on there for the latest license copy.
 */

class Controller extends Package
{
    protected $pkgHandle = 'jl_elephpant_info';
    protected $appVersionRequired = '8.5.2';
    protected $pkgVersion = '7.9.2';

    public function getPackageAutoloaderRegistries()
    {
        $auto = [
            'src/JlInfo' => '\\JlInfo',
        ];
        return $auto;
    }

    public function getPackageDescription()
    {
        return t('From dashboard pages run phpinfo() and view extended information about the Concrete CMS environment and configuration');
    }

    public function getPackageName()
    {
        return t('Ele-PHP-ant Info');
    }

    public function install()
    {
        $pkg = parent::install();
        $this->do_install($pkg);
    }

    public function upgrade()
    {
        parent::upgrade();
        $this->do_install($this);
    }

    private function do_install($pkg)
    {
        $this->install_pages($pkg);
    }

    /*
     * Install dashboard pages
     */
    private function install_pages($pkg)
    {
        $dashboard_pages = $this->list_dashboard_pages($pkg);
        foreach ($dashboard_pages as $path => $page_setup) {
            $page = Page::getByPath($path);
            if (!is_object($page) || $page->isError()) {
                $name = $page_setup['cName'];
                $description = $page_setup['cDescription'];
                $page = SinglePage::add($path, $pkg);
                $page->update(['cName' => $name, 'cDescription' => $description]);
            }
            $page->refreshCache();
        }
    }


    /*
    * List of dashboard single pages
    */
    public function list_dashboard_pages($pkg)
    {
        $dashboard_pages = [
            '/dashboard/system/environment/elephpant_info'
            => [
                'cName' => t('PHP Information'),
                'cDescription' => t('Run phpinfo() from the dashboard.')
            ],
            '/dashboard/system/environment/constants_info'
            => [
                'cName' => t('Constants Information'),
                'cDescription' => t('Constants defined by PHP, by the site environment and by Concrete CMS.')
            ],
            '/dashboard/system/environment/app_info'
            => [
                'cName' => t('Application Information'),
                'cDescription' => t('Application and assets information from the Concrete CMS environment.')
            ],
            '/dashboard/system/environment/config_info'
            => [
                'cName' => t('Config Information'),
                'cDescription' => t('Config information from the Concrete CMS environment and packages.')
            ],
            '/dashboard/system/environment/mysql_info'
            => [
                'cName' => t('MySQL Information'),
                'cDescription' => t('Config information from the MySQL database.')
            ]
        ];
        return $dashboard_pages;
    }

}
