<?php

namespace Concrete\Package\JlElephpantInfo\Controller\SinglePage\Dashboard\System\Environment;

use Concrete\Core\Page\Page;
use Concrete\Core\Routing\Redirect;
use Concrete\Core\Support\Facade\Application;
use Concrete\Core\Http\Request;
use Concrete\Core\Page\Controller\DashboardPageController;
use Concrete\Core\Support\Facade\Config;
use Illuminate\Support\Arr;

defined('C5_EXECUTE') or die('Access Denied');

/*
 * John Liddiard (aka JohntheFish) www.c5magic.co.uk
 * This software is licensed under the terms described in the concrete5.org marketplace.
 * Please find the add-on there for the latest license copy.
 */

class AppInfo extends DashboardPageController
{
    public function on_start()
    {

        if (is_callable('parent::on_start')) {
            parent::on_start();
        }
        if (empty($this->app)) {
            $this->app = Application::getFacadeApplication();
        }
        $this->set('app', $this->app);
        $this->addHeaderItem($this->some_css());

    }

    public function view()
    {
        $data = Request::post();

        $poss_info_cats = $this->list_poss_app_groups();
        $poss_info_cats['JTF_SUPPORT'] = t('Addon Support');
        $this->set('poss_info_cats', $poss_info_cats);

        /*
         * See what button was clicked. Should only be one category.
         * This is just an easy way to check through them in sequence
         */
        $info_cats = [];
        foreach ($data as $ky => $val) {
            if (preg_match("/ccm\-submit\-(\/dashboard\/system\/.*)/", $ky, $m)) {
                $page = Page::getByPath($m[1]);
                return Redirect::url($page->getCollectionLink());
            }
           if (preg_match("/ccm\-submit\-/i", $ky)) {
                $cat = array_search($val, $poss_info_cats);
               if(strpos($ky,'JTF_SUPPORT')!==false){
                   $info_cats[] = 'JTF_SUPPORT';
               }else if ($cat !== false) {
                    $info_cats[] = $cat;
                }
            }
        }

        if (empty($info_cats)) {
            $info_cats[] = 'aliases';
        }

        $selected_app_group = reset($info_cats);

        $this->set('selected_app_group', $selected_app_group);
        $this->set('selected_group_app', $this->get_app_group($selected_app_group, $data['dot_notation']));

    }

    /*
	List all app_groups
	*/
    private function list_poss_app_groups()
    {
        $all_app = Config::get('app');
        $all_app_groups = array_keys($all_app);
        sort($all_app_groups);
        $app_groups = array_combine($all_app_groups, $all_app_groups);
        return $app_groups;
    }

    /*
	List a specific group
	*/
    private function get_app_group($selected_group = 'aliases', $notation=null)
    {
        $all_app = Config::get('app');

        $app_group = $all_app[$selected_group];

        if(!empty($app_group) && is_array($app_group) && $notation==='dot'){
            $app_group = Arr::dot($app_group);
        }

        if (!empty($app_group) && is_array($app_group)) {
            uksort($app_group, function ($a, $b) {
                $a = trim(strtolower($a));
                $b = trim(strtolower($b));
                if ($a < $b) {
                    return -1;
                }
                if ($a > $b) {
                    return 1;
                }
                return 0;
            });
        }
        return $app_group;
    }


    /**
     * Some css to pretty up the output format
     */
    public function some_css()
    {
        ob_start();
        ?>
        <style type="text/css">
            .elephant-v9-header{
                display:none;
            }
            #info_data_notation{
                display:inline-block;
                margin-right:30px;
            }
            .elephant-info_support #table_filter_container,
            .elephant-aliases #info_data_notation,
            .elephant-facades #info_data_notation,
            .elephant-api_keys #info_data_notation,
            .elephant-debug #info_data_notation,
            .elephant-entity_namespaces #info_data_notation,
            .elephant-import_processors #info_data_notation,
            .elephant-importer_routines #info_data_notation,
            .elephant-namespace #info_data_notation,
            .elephant-package_items #info_data_notation,
            .elephant-providers #info_data_notation,
            .elephant-server_timezone #info_data_notation,
            .elephant-jtf_support #info_data_notation,
            .elephant-info_support #info_data_notation{
                display:none;
            }

            #app_info_form .ccm-dashboard-header-buttons .btn {
                margin-left: 5px;
                margin-bottom: 5px;
            }

            #app_info_form .app_header_menu .dropdown-menu {
                right: 0;
                left: auto;
            }

            #app_info_form tr > td:first-child, #app_info_form th {
                background: #EEE;
            }

            #app_info_form td.v {
                word-wrap: break-word;
                white-space: pre-wrap;
                word-break: break-all;
            }

            #app_info_form pre {
                word-wrap: break-word;
                white-space: pre-wrap;
                word-break: break-word;
            }

            #app_info_form h2 a, #app_info_form h2 a:hover {
                text-decoration: none;
                color: inherit;
            }

            #app_info_form ul {
                width: 800px;
                max-width: calc(100vw - 100px);
                column-count: 3;
                column-gap: 15px;
                column-rule: 1px solid #EEE;
            }

            #app_info_form ul li {
                clear: both;
            }

            #app_info_form ul li.info_first_extra {
                border-top: 1px solid #eee;
            }
        </style>
        <?php

        $css = ob_get_contents();
        ob_end_clean();
        return $css;
    }

}
