<?php

namespace Concrete\Package\JlElephpantInfo\Controller\SinglePage\Dashboard\System\Environment;

use Concrete\Core\Page\Page;
use Concrete\Core\Routing\Redirect;
use Concrete\Core\Support\Facade\Application;
use Concrete\Core\Http\Request;
use Concrete\Core\Page\Controller\DashboardPageController;
use Concrete\Core\Support\Facade\Config;
use Illuminate\Support\Arr;

defined('C5_EXECUTE') or die('Access Denied');

/*
 * John Liddiard (aka JohntheFish) www.c5magic.co.uk
 * This software is licensed under the terms described in the concrete5.org marketplace.
 * Please find the add-on there for the latest license copy.
 */

/*
 * https://documentation.concretecms.org/developers/framework/configuration-and-keyvalue-storage/storing-configuration-values
 */

class ConfigInfo extends DashboardPageController
{
    public function on_start()
    {

        if (is_callable('parent::on_start')) {
            parent::on_start();
        }
        if (empty($this->app)) {
            $this->app = Application::getFacadeApplication();
        }
        $this->fh = $this->app->make('helper/file');
        $this->set('app', $this->app);
        $this->addHeaderItem($this->some_css());

    }

    public function view()
    {
        $data = Request::post();

        $poss_info_cats = $this->list_poss_config_groups();
        $poss_info_cats['Config Database'] = t('Config Database');
        $poss_info_cats['JTF_SUPPORT'] = t('Addon Support');
        $this->set('poss_info_cats', $poss_info_cats);

        /*
         * See what button was clicked. Should only be one category.
         * This is just an easy way to check through them in sequence
         */
        $info_cats = [];
        foreach ($data as $ky => $val) {
            if (preg_match("/ccm\-submit\-(\/dashboard\/system\/.*)/", $ky, $m)) {
                $page = Page::getByPath($m[1]);
                return Redirect::url($page->getCollectionLink());
            }

            if (preg_match("/ccm\-submit\-/i", $ky)) {
                if(strpos($ky,'JTF_SUPPORT')!==false){
                    $info_cats[] = 'JTF_SUPPORT';
                }else if (!empty($poss_info_cats[$val])) {
                    $info_cats[] = $val;
                }else if (!empty($poss_info_cats[$val.'.php'])) {
                    $info_cats[] = $val.'.php';
                }
            }
        }

        if (empty($info_cats)) {
            $info_cats[] = 'concrete.php';
        }

        $selected_config_group = reset($info_cats);

        $this->set('selected_config_group', $selected_config_group);
        $this->set('selected_group_config', $this->get_config_group($selected_config_group, $data['dot_notation']));

    }

    /*
	List all config_groups
	*/
    private function list_poss_config_groups()
    {
        $all_config = $this->recursive_config_list();
        sort($all_config);
        $config_groups = array_combine(
            $all_config,
            array_map(
                function ($a) {
                    return preg_replace("/\.php$/",'', $a);
                },
                $all_config
            )
        );
        return $config_groups;
    }

    private function recursive_config_list()
    {
        $files = $this->fh->getDirectoryContents(DIR_CONFIG_SITE, ['doctrine'], true);

        if (empty($files)) {
            return;
        }
        $config_files = [];
        foreach ($files as $file) {
            $ext = $this->fh->getExtension($file);
            if ($ext !== 'php') {
                continue;
            }
            if (!strstr($file, '/generated_overrides/')) {
                continue;
            }
            $config_files[] = str_replace(DIR_CONFIG_SITE . '/generated_overrides/', '', $file);
        }
        return $config_files;
    }

    /*
	List a specific group
	*/
    private function get_config_group($selected_group = 'site', $notation=null)
    {
        if ($selected_group === 'Config Database'){
            $db = $this->app->make('database')->connection();
            $results = $db->fetchAll('SELECT * FROM `Config` ORDER BY `configNamespace` ASC, `configGroup` ASC, `configItem` ASC');
            $config_group = [];
            if(!empty($results)){
                foreach ($results as $item){

                    if(!empty($item['configNamespace'])){
                        $k = $item['configNamespace'].'::'.$item['configGroup'].'.'.$item['configItem'];
                    }else{
                        $k = $item['configGroup'].'.'.$item['configItem'];
                    }
                    $config_group[$k] = $item['configValue'];
                }
                return $config_group;
            }

        } else {
            try {
                $key = str_replace('.php','',str_replace('/','::',$selected_group));
                $config_group = Config::get($key);

                /*
                 * is the equivalent of
                 * $config_group = Arr::dot(include DIR_CONFIG_SITE . '/generated_overrides/' . $selected_group);
                 */

                if(!empty($config_group) && is_array($config_group) && $notation==='dot'){
                    $config_group = Arr::dot($config_group);
                }

                if (!empty($config_group) && is_array($config_group)) {
                    uksort($config_group, function ($a, $b) {
                        $a = trim(strtolower($a));
                        $b = trim(strtolower($b));
                        if ($a < $b) {
                            return -1;
                        }
                        if ($a > $b) {
                            return 1;
                        }
                        return 0;
                    });
                }
                return $config_group;
            } catch (\Exception $x) {
                return ['Exception' => $x->getMessage()];
            } catch (\Throwable $x) {
                return ['Throwable' => $x->getMessage()];
            }
        }
    }


    /**
     * Some css to pretty up the output format
     */
    public function some_css()
    {
        ob_start();
        ?>
        <style type="text/css">
            .elephant-v9-header{
                display:none;
            }
            #info_data_notation{
                display:inline-block;
                margin-right:30px;
            }
            .elephant-info_support #table_filter_container,
            .elephant-config_database #info_data_notation,
            .elephant-jtf_support #info_data_notation,
            .elephant-info_support #info_data_notation{
                display:none;
            }

            #config_info_form .ccm-dashboard-header-buttons .btn {
                margin-left: 5px;
                margin-bottom: 5px;
            }

            #config_info_form .config_header_menu .dropdown-menu {
                right: 0;
                left: auto;
            }

            #config_info_form tr > td:first-child, #config_info_form th {
                background: #EEE;
            }

            #config_info_form td.v {
                word-wrap: break-word;
                white-space: pre-wrap;
                word-break: break-all;
            }

            #config_info_form pre {
                word-wrap: break-word;
                white-space: pre-wrap;
                word-break: break-word;
            }

            #config_info_form h2 a, #config_info_form h2 a:hover {
                text-decoration: none;
                color: inherit;
            }

            #config_info_form ul {
                width: 800px;
                max-width: calc(100vw - 100px);
                column-count: 3;
                column-gap: 15px;
                column-rule: 1px solid #EEE;
            }

            #config_info_form ul li {
                clear: both;
            }

            #config_info_form ul li.info_first_extra {
                border-top: 1px solid #eee;
            }
        </style>
        <?php

        $css = ob_get_contents();
        ob_end_clean();
        return $css;
    }

}
