<?php

namespace Concrete\Package\JlElephpantInfo\Controller\SinglePage\Dashboard\System\Environment;

use Concrete\Core\Page\Page;
use Concrete\Core\Routing\Redirect;
use Concrete\Core\Support\Facade\Application;
use Concrete\Core\Http\Request;
use Concrete\Core\Page\Controller\DashboardPageController;

defined('C5_EXECUTE') or die('Access Denied');

/*
 * John Liddiard (aka JohntheFish) www.c5magic.co.uk
 * This software is licensed under the terms described in the concrete5.org marketplace.
 * Please find the add-on there for the latest license copy.
 */

class ConstantsInfo extends DashboardPageController
{
    public function on_start()
    {

        if (is_callable('parent::on_start')) {
            parent::on_start();
        }
        if (empty($this->app)) {
            $this->app = Application::getFacadeApplication();
        }
        $this->set('app', $this->app);
        $this->addHeaderItem($this->some_css());

    }

    public function view()
    {
        $data = Request::post();

        $poss_info_cats = $this->list_poss_constant_groups();
        $poss_info_cats['JTF_SUPPORT'] = t('Addon Support');
        $this->set('poss_info_cats', $poss_info_cats);

        /*
         * See what button was clicked. Should only be one category.
         * This is just an easy way to check through them in sequence
         */
        $info_cats = [];
        foreach ($data as $ky => $val) {
            if(preg_match("/ccm\-submit\-(\/dashboard\/system\/.*)/", $ky, $m)){
                $page = Page::getByPath($m[1]);
                return Redirect::url($page->getCollectionLink());
            }
            if (preg_match("/ccm\-submit\-/i", $ky)) {
                $cat = array_search($val, $poss_info_cats);
                if(strpos($ky,'JTF_SUPPORT')!==false){
                    $info_cats[] = 'JTF_SUPPORT';
                }else if ($cat !== false) {
                    $info_cats[] = $cat;
                }
            }
        }

        if (empty($info_cats)) {
            $info_cats[] = 'user';
        }

        $selected_constant_group = reset($info_cats);

        $this->set('selected_constant_group', $selected_constant_group);
        $this->set('selected_group_constants', $this->get_constant_group($selected_constant_group));

    }

    /*
	List all constant_groups
	*/
    private function list_poss_constant_groups(){
        $all_constant_groups = array_keys(get_defined_constants(true));
        sort($all_constant_groups);
        $constant_groups = array_combine($all_constant_groups,$all_constant_groups);
        // note this hash is inverted
        unset($constant_groups['user']);
        return array_merge(
                [
                        'user' => t('Concrete CMS'),
                        'all' => t('All'),
                ],
                $constant_groups
        );
    }

    /*
	List a specific group
	*/
    private function get_constant_group($selected_group = 'user'){
        $all_const = get_defined_constants(true);
        $this->add_special_conatants($all_const);
        $poss_constant_groups = $this->list_poss_constant_groups();

        // spacial case of 'All'
        if ($poss_constant_groups[$selected_group] === 'All' || $poss_constant_groups[$selected_group] === t('All')){
            $flattend_const = array();
            foreach ($all_const as $acky=>$acval){
                foreach ($acval as $k2=>$v2){
                    $flattend_const[$k2.' ['.$acky.']'] = $v2;
                }
            }
            ksort($flattend_const);
            return $flattend_const;
        }

        $constant_group = $all_const[$selected_group];

        if(!empty($constant_group)){
            uksort($constant_group, function($a,$b){
                $a = trim(strtolower($a));
                $b = trim(strtolower($b));
                if ($a<$b){
                    return -1;
                }
                if ($a>$b){
                    return 1;
                }
                return 0;
            });
        }


        return $constant_group;
    }

    private function add_special_conatants(&$all_const){
        $all_const['user']['getcwd()'] = getcwd();
    }


    /**
     * Some css to pretty up the output format
     */
    public function some_css()
    {
        ob_start();
        ?>
        <style type="text/css">
            .elephant-v9-header{
                display:none;
            }
            #info_data_notation{
                display:inline-block;
                margin-right:30px;
            }
            .elephant-info_support #table_filter_container,
            .elephant-jtf_support #info_data_notation,
            .elephant-info_support #info_data_notation{
                display:none;
            }

            #constants_info_form .ccm-dashboard-header-buttons .btn{
                margin-left:5px;
                margin-bottom:5px;
            }
            #constants_info_form .constants_header_menu .dropdown-menu {
                right:0;
                left:auto;
            }
            #constants_info_form tr > td:first-child, #constants_info_form th
            {
                background: #EEE;
            }
            #constants_info_form td.v{
                word-wrap: break-word;
                white-space: pre-wrap;
                word-break: break-all;
            }
            #constants_info_form pre{
                word-wrap: break-word;
                white-space: pre-wrap;
                word-break: break-word;
            }
            #constants_info_form h2 a, #constants_info_form h2 a:hover{
                text-decoration:none;
                color: inherit;
            }
            #constants_info_form ul{
                width: 800px;
                max-width: calc( 100vw - 100px );
                column-count:3;
                column-gap:15px;
                column-rule:1px solid #EEE;
            }
            #constants_info_form ul li{
                clear:both;
            }
            #constants_info_form ul li.info_first_extra{
                border-top:1px solid #eee;
            }
        </style>
        <?php

        $css = ob_get_contents();
        ob_end_clean();
        return $css;
    }

}
